import 'dart:collection';

import 'package:flumen_encodable/src/encodable.dart';
import 'package:flumen_encodable/src/coding.dart';
import 'package:flumen_encodable/src/keyed_archive.dart';
import 'package:flumen_encodable/src/resolver.dart';

/// A list of values in a [KeyedArchive].
///
/// This object is a [List] that has additional behavior for encoding and decoding [Coding] objects.
class ListArchive extends Object
    with ListMixin<dynamic>
    implements Referencable {
  final List<dynamic> _inner;

  ListArchive() : _inner = [];

  /// Replaces all instances of [Map] and [List] in this object with [KeyedArchive] and [ListArchive]s.
  ListArchive.from(List<dynamic> raw)
      : _inner = raw.map((e) {
          if (e is Map) {
            return KeyedArchive(e.cast<String, dynamic>());
          } else if (e is List) {
            return ListArchive.from(e);
          }
          return e;
        }).toList();

  @override
  dynamic operator [](int index) => _inner[index];

  @override
  int get length => _inner.length;

  @override
  set length(int length) {
    _inner.length = length;
  }

  @override
  void operator []=(int index, dynamic val) {
    _inner[index] = val;
  }

  @override
  void add(dynamic element) {
    _inner.add(element);
  }

  @override
  void addAll(Iterable<dynamic> iterable) {
    _inner.addAll(iterable);
  }

  List<dynamic> toPrimitive() {
    return _inner.map((val) {
      if (val is KeyedArchive || val is ListArchive) {
        return val.toPrimitive();
      }
      return val;
    }).toList();
  }

  @override
  void resolveOrThrow(ReferenceResolver coder) {
    for (final i in _inner) {
      if (i is KeyedArchive) {
        i.resolveOrThrow(coder);
      } else if (i is ListArchive) {
        i.resolveOrThrow(coder);
      }
    }
  }
}
